#ifndef WIMESH_UTILS
#define WIMESH_UTILS


#include <sstream>
#include <iostream>
#include <typeinfo>
#include "rndvars.h"

using namespace std;


/*
 * Reads any type of value while skipping blank lines
 * and lines beginning with #
 * Throws an exception in case of failures
 */
template <class T>
void readValue (istream& is, T& t, const char* msg)
{
  string s;
  
  is >> s;
  
  while (s.c_str()[0] == '#')
  {
    getline (is, s);
    is >> s;
  }
  
  // This code safely converts from string
  stringstream myStream(s);

  if (myStream >> t)
     return;

  cerr << "Error while reading " << msg << endl;
  // Terminate the program
  throw;
}



/*
 * Helper class to read params from an XML config file (see parsers.cpp for examples)
 */
class ReadParams
{
public:
  ReadParams (string n, string v) : name(n), value(v), found(false) {}

  template <class vartype>
  void read (string param, vartype& var);
  bool notFound () { return !found; }

private:
  void readDouble (void* pvar) { *((double*)pvar) = atof (value.c_str()); }
  void readFloat (void* pvar) { *((float*)pvar) = atof (value.c_str()); }
  void readInt (void* pvar) { *((int*)pvar) = atoi (value.c_str()); }
  void readUInt (void* pvar) { *((unsigned int*)pvar) = atoi (value.c_str()); }
  void readLong (void* pvar) { *((long int*)pvar) = atol (value.c_str()); }
  void readString (void* pvar) { *((string*)pvar) = value; }
  void readBool (void* pvar) 
  {
    if (value == "true" || value == "1")
      *((bool*)pvar) = true;
    if (value == "false" || value == "0")
      *((bool*)pvar) = false;
  }
  void readRndVar (void* pvar) { stringstream ss(value); *((RndVar**)pvar) = new RndVar (ss); }

  string name;
  string value;
  bool found;
};


template <class vartype>
void ReadParams::read (string param, vartype& var)
{
  if (param != name)
    return;

  found = true;

  if (typeid(var).name() == typeid(double).name())
    readDouble (&var);
  else if (typeid(var).name() == typeid(float).name())
    readFloat (&var);
  else if (typeid(var).name() == typeid(int).name())
    readInt (&var);
  else if (typeid(var).name() == typeid(unsigned int).name())
    readUInt (&var);
  else if (typeid(var).name() == typeid(long int).name())
    readLong (&var);
  else if (typeid(var).name() == typeid(string).name())
    readString (&var);
  else if (typeid(var).name() == typeid(bool).name())
    readBool (&var);
  else if (typeid(var).name() == typeid(RndVar*).name())
    readRndVar (&var);
  else
    cerr << endl << "** Unsupported type (" << typeid(var).name() << ") in ReadParams::read !! **" << endl << endl;
}





#endif
