#include "rndvars.h"
#include <sstream>
#include <iterator>
#include "WMutils.h"
#include <sys/time.h>
#include <cmath>

RndVar::RndVar(istream& is)
{
	readDistro (is);

	vector<char>::iterator _distros = distros.begin();
	vector<double>::iterator _params = params.begin();
	vector<double> seqvalues;
	stringstream ss;
  
	/* the first char of distros is the number of distributions to be mixed */
	int num = distros[0] - '0';
	_distros++;
	
	if (num > 1)
		ss << distros[0] << " ";

	Random** Var=(Random**)malloc(num*sizeof(Random*));
	Constant** C=(Constant**)malloc(num*sizeof(Constant*));
	Uniform** U=(Uniform**)malloc(num*sizeof(Uniform*));
	Exponential** E=(Exponential**)malloc(num*sizeof(Exponential*));
	Sequence** S=(Sequence**)malloc(num*sizeof(Sequence*));

	for (int i=0; i<num; i++)
	{
		switch(*(_distros++))
		{
			case 'C':
				  C[0] = new Constant(*_params);
				  Var[i] = (Random*)C[0];
				  ss << "C " << *_params << " ";
				  _params++; C++;
				  break;
			case 'U': 
				  U[0] = new Uniform;
				  Var[i] = (Random*)new SumRandom((*(_params+1)-*_params) * (**U) + *_params);
				  ss << "U " << *_params << " " << *(_params+1) << " ";
				  _params+=2; U++;
				  break;
			case 'E':
				  E[0] = new Exponential;
				  Var[i] = (Random*)new SumRandom(*_params*(**E));
				  ss << "E " << *_params << " ";
				  _params++; E++;
				  break;
			case 'S':
				  ss << "S " << *_params << " ";
				  seqvalues.resize(*_params);
				  copy(_params+1,_params+1+ *_params, seqvalues.begin());
				  copy(_params+1,_params+1+ *_params, ostream_iterator<double>(ss," "));
				  S[0] = new Sequence(seqvalues);
				  Var[i] = (Random*)S[0];
				  _params += *_params; S++;
				  break;
		}
		if (num > 1)
			ss << probs[i] << " ";
	}

	name = ss.str();

	if (num==1)
		rv = Var[0];
	else
		rv = (Random *)new MixedRandom(num,probs,Var);
}


// RndVar::RndVar(const RndVar& rndvar)
// {
//   distros = new char[strlen(rndvar.distros)+1];
//   strcpy(distros, rndvar.distros);
//   
//   params = new double;
//   *params = *(rndvar.params);
// 
//   probs = new double;
//   *probs = *(rndvar.probs);
// }

double RndVar::getSeed ()
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return (0.49 * sin((double)tv.tv_usec) + 0.50);    //  0.01 <= seed <= 0.99
}



void RndVar::print(ostream& os)
{
	int d=1,p=0,num=distros[0]-'0';
	
	for (int i=0; i<num; i++)
	{
		switch(distros[d++])
		{
			case 'C':
				  os << "C(" << params[p++] << ")";
				  break;
			case 'U':
				  os << "U(" << params[p] << "," << params[p+1] << ")";
				  p+=2;
				  break;
			case 'E':
				  os << "Exp(1/" << params[p++] << ")";
				  break;
			case 'S':
				  os << "S(";
				  copy(params.begin()+p+1, params.begin()+p+params[p], ostream_iterator<double>(os,","));
				  p+=params[p];
				  os << params[p++] << ")";
				  break;
		}
		if (num>1)
		{
			os << "[" << probs[i] << "]";
			if (i<num-1)
			  os << "+";
		}
	}
}


/* read the parameters of a random variable from an istream */
void RndVar::readDistro(istream& is)
{
	char c, msg[]="Unexpected value or EOF while reading random variable distributions";
	int num=1,len;      /* by default, no mixture */
	double d;

	readValue (is, c, msg);

	/* if 1 < c <= 9 then it is a mixture of distributions */
	if (c>'1' && c<='9')
		num=c-'0';

	distros.push_back((num>1 ? c : '1'));

	probs = (num>1 ? new double[num] : 0);

	if (num==1)
		is.unget();

	for (int i=0; i<num; i++)
	{
		readValue (is, c, msg);
		if (c!='C' && c!='U' && c!='E' && c!='S')
		{
			cerr << "Only (C)onstant, (U)niform, (E)xponential and (S)equence distributions are currently supported" << endl;
			throw;
		}
		distros.push_back(c);
		switch (c)
		{
			case 'C':        // constant
				  readValue (is, d, msg);    /* value */
				  params.push_back(d);
				  break;
			case 'U':        // U(min,max)
				  readValue (is, d, msg);   /* min */
				  params.push_back(d);
				  readValue (is, d, msg);   /* max */
				  params.push_back(d);
				  break;
			case 'E':        // Exp(1/mean)
				  readValue (is, d, msg);   /* mean */
				  params.push_back(d);
				  break;
			case 'S':       // Sequence
				  readValue(is, len, msg);   /* number of elements */
				  params.push_back(len);
				  for (int j=0; j<len; j++)
				  {
				    readValue(is, d, msg);   /* value */
				    params.push_back(d);
				  }
				  break;
		}
		if (num>1)               /* read probability */
			readValue (is, probs[i], msg);
	}
}


const char* Sequence::Name()  { return "Sequence"; }
