#ifndef SUB_GRAPH
#define SUB_GRAPH


#include "WMtypes.h"
#include "GraphElements.h"
#include "Graph.h"


/*
 * This class is just defined to create an empty file to pass to the Graph constructor,
 * before the Graph constructor is called
 */
class EmptyGraph
{
public:
  EmptyGraph () { m_ofs.open (".WMemptygraph", ofstream::out); m_ofs.close (); }
private:
  ofstream m_ofs;
};

/*
 * SubGraph is a class derived from Graph (ksp_2.0) to add:
 * - a constructor to build a graph from a Topology*
 * - methods to remove an edge or a vertex from a graph
 * - a (more) correct copy constructor
 * - a (more) correct destructor
 */
class SubGraph : public EmptyGraph, public Graph {
public:
  SubGraph(Topology* T, bool reversed = false);

  SubGraph (const SubGraph& graph);

  ~SubGraph ();

  void removeEdge (int u, int v);

  void removeVertex (int u);

  int getNumVertices (void) { return m_nVertexNum; }

  void pruneEdges (linkPruneFunc* prune);

  void setLinkCosts (linkCostFunc* cost);

private:
  Topology* m_topology;
  bool m_reversed;
};


#endif
