#include "wimesh-ns3-parser.h"
#include <ctime>

using namespace std;

/*
 * Usage example:
 * 
 * cat configNs3.xml | ./RunNs3
 * 
 * This program runs an ns-3 simulation starting from a serialized topology file
 * created by WMrun. Various parameters can be configured in the XML config file,
 * including the routing protocol, the traffic load and possible failure events.
 * A flow monitor and an energy meter can be enabled, which measure, respectively,
 * network parameters (throughput, delay, packet loss) and node power consumption.
 * Such values are stored in file with names starting, respectively, with "flow_"
 * and "energy_".
 */


int main()
{
  WiMeshNs3Parser parser;

  try
  {
    parser.set_substitute_entities(true);
    parser.parse_stream(std::cin);
  }
  catch(const xmlpp::exception& ex)
  {
    std::cerr << "libxml++ exception: " << ex.what() << std::endl;
  }
  
  WiMeshNs3* ns3 = parser.GetNs3Ptr();
  
  if (!ns3)
  {
    cerr << endl << "*** ERROR: No valid Routing element specified ***" << endl << endl;
    return -1;
  }

  if (!ns3->GetTopology())
  {
    cerr << endl << "*** ERROR: No valid Topology element specified ***" << endl << endl;
    return -1;
  }

  clock_t startt = clock();

  ns3->RunSimulation();

  double elapsed_time = (double)(clock() - startt) / CLOCKS_PER_SEC;

  cout << endl << "*** Running the ns-3 simulation took " << elapsed_time << " seconds ***" << endl << endl;

  // That's all...
}

